function classifyPatterns(featPaths, statsData, classes, tissueLabels, writepath, method)

% Copyright (C) 2006  Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

doofus = 1;
if doofus == 1
    if exist( writepath,'file')
        return
    end

    prp = writepath;
    prp((prp == '/') | (prp == '.')) = '_';
    pr = ['tmp/classifying_' prp '.txt'];
    if ~exist( pr,'file')
        fr = fopen( pr, 'w');
    else
        return
    end

    if length(featPaths)~=length(classes)
        fclose(fr);
        delete(pr);
        error( 'check input values')
    end
end

% 1. Load data
s_fp = zeros(length(featPaths),1);
for i=1:length(featPaths)
    s_fp(i) = size(featPaths{i},2);
end
su_fp = sum(s_fp);
cs_fp = [0; cumsum( s_fp)]; cs_fp( end) = [];

data = zeros( su_fp, 841);
tlabels = zeros( su_fp, 1);
clabels = zeros( su_fp, 1);
alabels = zeros( su_fp,1);
stats = zeros( su_fp, 3);
images = []; images{su_fp} = [];
for i=1:length(featPaths)
    for j=1:s_fp(i)
        ind = cs_fp(i)+j;
        fid=fopen( statsData{i}{j},'r');  stat = fread(fid);  fclose(fid);
        stat(end)=[];  stat=char(stat)';  stat=str2num(stat);
        load( featPaths{i}{j});
        data(ind,:) = features;
        tlabels(ind) = tissueLabels{i}(j);
        clabels(ind) = i;
        stats(ind,:) = stat;
        images(ind) =  {[statsData{i}{j}(1:end-3) 'jpg']};
        idx_abID = find( statsData{i}{j}=='/');
        alabels(ind) = str2num( statsData{i}{j}(idx_abID(2)+1:idx_abID(3)-1));
    end
end

% 2. Remove bad images based on features/labels

% Remove cyan staining and images with too much black in them.
ind = find(stats(:,1)>13);
data(ind,:) = [];
stats(ind) = [];
clabels(ind) = [];
tlabels(ind) = [];
alabels(ind) = [];
images(ind) = [];


% 3. Remove bad features
data = data(:,[2:end]);

% 4. Remove cases where there is only one tissue/location combination
sampleID = str2num([num2str(clabels) num2str(tlabels)]);
[c b] = hist( sampleID, [1:1:max(sampleID)]);
idx = find(c==1);

ind = zeros(size(idx));
for i=1:length(idx)
    ind(i) = find(sampleID==idx(i));
end

data(ind,:) = [];
stats(ind) = [];
clabels(ind) = [];
tlabels(ind) = [];
alabels(ind) = [];
images(ind) = [];


% 5. Data partitioning
% Done in the even tissues/class then even antibodies/class approach
u_c = unique( clabels);
[u_Abs I J] = unique(alabels);
bool = 0;
ttlist = zeros( size( clabels));
for i=1:length(u_c)
    idx = find( clabels==u_c(i));
    s_tlabels = tlabels(idx);
    s_alabels = J(idx);
    
    [c_t u_t] = hist( s_tlabels, 1:1:65);
    u_t = u_t(c_t>0);
    
    for j=1:length(u_t)
        idx2 = find(s_tlabels==u_t(j));
        s2_alabels = s_alabels(idx2);
        [c_a u_a] = hist( s2_alabels, 1:1:max(u_Abs));
        u_a = u_a(c_a>0);
        
        for k=1:length(u_a)
            idx3 = find(s2_alabels==u_a(k));
            my_idx = mod(bool:bool+length(idx3)-1,2)+1;
            if mod( length(idx3), 2)
                bool = mod(bool + 1,2);
            end
            ttlist(idx(idx2(idx3))) = my_idx;
        end
    end
end

trainlabels = clabels(ttlist==1);
testlabels = clabels(ttlist==2);
trainlabels_tissue = tlabels(ttlist==1);
testlabels_tissue = tlabels(ttlist==2);
trainlabels_antibody = alabels(ttlist==1);
testlabels_antibody = alabels(ttlist==2);
trainimages = images(ttlist==1);
testimages = images(ttlist==2);

traindata = data(ttlist==1,:);
testdata = data(ttlist==2,:);

% 6. Feature selection by SDA
[traindata,testdata] = featnorm( traindata, testdata);
traindata = double( traindata*2-1);
testdata = double(testdata*2-1);

u = unique( trainlabels);
feat = []; feat{length(u)} = [];
for i=1:length(u)
    feat{i} = traindata( trainlabels==u(i), :);% 5:end);
end
logfilename = [writepath '_sdalog.txt'];
idx_sda = ml_stepdisc( feat,logfilename);

% 7. Classification
[model,options] = trainClassifier( trainlabels,traindata(:,idx_sda));
[predlabels accuracy weights] = svmpredict( testlabels,testdata(:,idx_sda), model,'-b 1');

[C U D W] = conmatrix( testlabels, predlabels);

disp( 'conmatrix:');
disp(D);
disp( 'accuracy:');
disp(W);
disp(classes);

save( writepath, 'model','options','idx_sda','predlabels','weights',...
    'trainlabels','testlabels','traindata','testdata','classes',...
    'trainlabels_tissue','testlabels_tissue',...
    'trainlabels_antibody','testlabels_antibody',...
    'trainimages','testimages');

if doofus ==1
    fclose(fr);
    delete(pr);
end

return
