function [C,U,D,W] = lrclassif( all_features)
% function [D,W] = classif( all_features,numit)
% FUNCTION CLASSIF( ALL_FEATURES)
% 
% Inputs: ALL_FEATURES - the cell-structure
% Outputs: D - confusion matrix, in %
% 	   W - the weighted accuracy, in %

% Copyright (C) 2006  Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

eta = 1e-3;
slack = .4;

if ~exist( 'numit','var')
    numit = 10000;
end

% Split the data into training and testing sets
[train test] = splitt( all_features, 7/10);

% Reorganize data back into the original format
[traindata,trainlabels] = featcell2mat( train);
[testdata,testlabels] = featcell2mat( test);

% Normalize features so each feature (column) has min 
%  value of 0 and max value of 1
[traindata,testdata] = featnorm( traindata,testdata);

lrlabels=[]; nnlabels=[]; svmlabels=[];

disp( 'Logistic Regression');
% Train a classifier, w. Type help lrtrain for more
w = lrtrain( traindata,trainlabels, eta, slack, numit);

% Report training accuracy
clabels = lrtest( traindata, w);
[C U D W] = conmatrix( trainlabels, clabels);
disp( ['Training accuracy is ' num2str(W*100) '%']);

% Apply classifier on testing set, report testing accuracy
lrlabels = lrtest( testdata, w);
[C U D W] = conmatrix( testlabels, lrlabels);
disp( ['Testing accuracy is ' num2str(W*100) '%']);
