function w = lrtrain( data, label, eta, lambda, maxiter);
% W = LRTRAIN( DATA, ETA, LAMBDA, MAXITER);
%    Train a logistic regression classifier. X is the MxN
%    feature matrix where M is the number of images and N
%    is the number of features. Y is a Mx1 matrix 
%    containing the data labels which can take on values 
%    0 or 1. 
%
%    Optional parameters are ETA, LAMBDA, and MAXITER (the 
%    number of iterations that will be used). These 
%    parameters are set by default to 0.01, 0, and 200, 
%    respectively. 
%
% Justin Newberg

% Copyright (C) 2006  Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

if ~exist( 'eta','var')
    eta = 10^-4;
end
if ~exist( 'lambda','var')
    lambda = 0;
end
if ~exist( 'maxiter','var')
    maxiter = 200;
end

K = length( unique( label));
Km = K - 1;

w = zeros( size(data,2)+1,Km);

x = [ones(size(data,1),1) data];

labels = zeros( size(label,1), Km);
for i=1:Km
	labels(:,i) = i==label;
end

for count=1:maxiter
	if ~mod(count,floor(maxiter/10)) 
		disp([ num2str(count) '/' num2str(maxiter)]); 
	end

	e = exp(x*w);
	P = e./repmat( 1 + sum(e,2), [1 Km]);
	w = w + eta*(x'*(labels - P) - lambda*w);
end

return
