function splits = partition( labels, N, rn)
% SPLITS = PARTITION( LABELS, N)
%   a fold partitioning function
%   outputs splits is a cell array with N entries

% Copyright (C) 2006  Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

if ~exist( 'rn','var')
    rn = 13;
end
rand( 'seed',rn);

% 1. set number of folds
[c b] = hist( labels,1:1:max(labels));
idx = find(c==0);
c(idx) = [];
b(idx) = [];
mi = min(c);

if ~exist( 'N','var')
    N = 10;
end
if mi<N
    N=mi;
end

% 2. split dataset into N folds
splits = [];
splits{N} = [];
for i=1:length(b)
    idx = find( labels==b(i));
    rp = randperm(length(idx));  % randomize partitioning
    val = floor(length(idx)/N);
    array = val+zeros(N,1);
    re = length(idx)-val*N;
    array(1:re) = array(1:re)+1;
    st = 1;
    for j=1:N
        en = st+array(j);
        splits{j} = [splits{j}; idx(rp(st:en-1))];
        st = en;
    end
end
