function [model,options] = trainClassifier( trainlabels, traindata, c, g)
% TRAINCLASSIFIER   Train a classifier!
%    [MODEL OPTIONS] = TRAINCLASSIFIER( TRAINLABELS, TRAINDATA)
%    trains a SVM classifier (MODEL) on the training data
%    (TRAINDATA) using training labels (TRAINLABELS). OPTIONS
%    is a string that reports what parameters were set for
%    model training. Parameter tuning of C and gamma is done
%    through a simple grid search over a set range.
%
% Last modified Justin Newberg 14 April 2008

% Determine/specify number of folds

% Copyright (C) 2006  Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

wopt = ' ';
[U I J] = unique(trainlabels);
[stot b] = hist(J,1:1:max(J));
wtot = min(stot)./stot;
for j=1:length(wtot)
    wopt = [wopt '-w' num2str(U(j)) ' ' num2str(wtot(j)) ' '];
end
% wopt = ' ';

N = 10;

% Brute force grid search
ma = 0;
%fprintf( [num2str(length(c)) ':']);
% g = 1;%%%%%
for i=1:length(c),
    tic;
    for j=1:length(g),
%         options = ['-s 0 -m 1000 -t 0 -c ' num2str(c(i)) ' -b 0 -v ' num2str(N) wopt];%%%%%
        options = ['-s 0 -m 1000 -t 2 -c ' num2str(c(i)) ' -g ' num2str(g(j)) ' -b 1 -v ' num2str(N) wopt];
        acc_cv = svmtrain( trainlabels, traindata, options);

        if acc_cv>ma
            ma = acc_cv;
%             boptions = ['-s 0 -m 1000 -t 0 -c ' num2str(c(i)) ' -b 1' wopt];%%%%%
            boptions = ['-s 0 -m 1000 -t 2 -c ' num2str(c(i)) ' -g ' num2str(g(j)) ' -b 1' wopt];
        end

    end
%    fprintf([' ' num2str(i) ' ']);
%    toc
end
%fprintf(char(10));
options = boptions;
model = svmtrain( trainlabels, traindata, options);

return
