function [ofeat,oname] = features_nuclearOverlap( prot, dna)
% Takes background subtracted images

% Copyright (C) 2006  Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

% Threshold by Otsu's method
thr_prot = graythresh(prot)*255;
thr_dna = graythresh(dna)*255;

obj_prot = prot>thr_prot;
obj_dna = dna>thr_dna;

area_prot = sum(obj_prot(:));
area_dna = sum(obj_dna(:));

overlap = obj_prot & obj_dna;
areaOverlap = sum(overlap(:));
SLFoverlapRatio = areaOverlap / area_prot;

SLFoverlapIntRatio = sum(prot(overlap)) / sum(prot(:));

dist_dna = bwdist(obj_dna);
SLFdistance = sum(dist_dna(obj_prot))/area_prot;

ofeat = [SLFoverlapRatio SLFoverlapIntRatio SLFdistance];
oname = {'SLFoverlapRatio','SLFoverlapIntRatio','SLFdistance'};

return

