function feat = ml_histfeatures( I, inputs)
% FEAT = ML_HISTFEATURES( I, GLEVELS)  Calculate multi-threshold features from a histogram
%   Vars:       I: a uint8 image, already stretched to GLEVELS
%          INPUTS: must contain
%                 .GLEVELS: the number of graylevels
%                 .ISRAWIM: lets code know whether or not to bg-subtract and
%                     stretch image ('yes','no')
%            FEAT: n features (where n is GLEVELS-2)
% 
% Justin Newberg, last edited 14 August 2008

% Copyright (C) 2006  Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

GLEVELS = inputs.GLEVELS;

if strcmp(inputs.ISRAWIM,'yes')
    [c b] = imhist( I);
    [a ind] = max(c);
    I=I-b(ind);

    sc = (GLEVELS-1) / double(max(I(:)));
    I = uint8(round(sc*double(I)));
end

I(I==0) = [];
len = length(I);

c = imhist(I);
c = c(2:GLEVELS);

csum = cumsum(c)';
feat = (len-csum(1:end-1))/len;

return
