function [organism, fusionClasses, pval, numims, acc, confmat] = analyzeSet(GLEVELS, tempvar)

% Copyright (C) 2006  Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


%% INITIALIZING VARIABLES AND ENVIRONMENT
setup
if ~exist('GLEVELS','var')
    GLEVELS = 64;
end


%% As an example, analyze set 1 (just because it has fewest images so is quickest as an example)
if ~exist('tempvar','var')
    tempvar=1;
end

% Set dataset loading parameters
% disp([GLEVELS tempvar]);
switch tempvar
    case 1
        % Vesicle SNF images
        organism = 'S. cerevisiae';
        fusionClasses = {'Snf1','Snf4 (vector)'};
        rootpath = 'data/Vesicle Capture Images';
        folders = {...
            [rootpath '/SNF1GFP x Empty'], ...
            [rootpath '/SNF1GFP x SNF4SNF7'] ...
            };
        classes = {'empty','bait'};
        wcard = '*.png';
    case 2
        % Vesicle images HOG
        organism = 'S. cerevisiae';
        fusionClasses = {'Pbs2','Hog1 (vector)'};
        rootpath = 'data/Vesicle Capture Images';
        folders = {...
            [rootpath '/PBS2GFP x Empty'], ...
            [rootpath '/PBS2GFP x HOG1SNF7'] ...
            };
        classes = {'empty','bait'};
        wcard = '*.png';
    case 3
        organism = 'C. albicans';
        fusionClasses = {'Snf1','Snf4 (vector)'};
        rootpath = 'data/Vesicle3';
        folders = { ...
            [rootpath '/SNF1.GFPXSNF4.SNF7/GFP Images'], ...
            [rootpath '/SNF1.GFPXSNF7 EMPTY VECTOR/GFP Images'] ...
            };
        classes = {'bait','empty'};
        wcard = '*.png';
    case 4
        organism = 'C. albicans';
        fusionClasses = {'Pbs2','Hog1 (vector)'};
        rootpath = 'data/Vesicle3';
        folders = { ...
            [rootpath '/PBS2.GFPXHOG1.SNF7/GFP Images'], ...
            [rootpath '/PBS2.GFPXSNF7 EMPTY VECTOR/GFP Images'] ...
            };
        classes = {'bait','empty'};
        wcard = '*.png';
    otherwise
        error('improper situation');
end

[Uu Iu Ju] = unique(classes);


% Load the data, and calculate features on each image as it is loaded
allfeats = [];
alllabels = [];
allfields = [];
uniquelabels = [];
classMax = zeros(size(Uu));
counter = 1;
for i=1:length(folders)
    d1 = dir([folders{i} '/' wcard]);
    feats = [];
    fields = [];
    labels = zeros(length(d1),1)+Ju(i);
    for j=1:length(d1)
        I = imread([folders{i} '/' d1(j).name]);
        I = I(:,:,1);
        feat = calcVesicleFeats( I, GLEVELS, classMax(Ju(i)));
        feats = [feats; feat];
        fields = [fields; zeros(size(feat,1),1)+counter];
        counter = counter + 1;
    end
    allfeats = [allfeats; feats];
    alllabels = [alllabels; zeros(length(fields),1) + Ju(i)];
    allfields = [allfields; fields];
    uniquelabels = [uniquelabels; labels];
end
uniquefields = unique(allfields);


pval = compareClasses( alllabels, allfeats);

% LOOCV (leave-one-out cross-validation) setup
splits = [];
for i=1:length(uniquelabels),
    splits{i} = i;
end
[trainidx,testidx] = partedsets( splits);

disp( ['Using ' num2str(length(trainidx)) ' folds']);

UU = unique(alllabels);

%% LOOCV
c = 2.^(-4:1:8);
g = 2.^(-8:1:8);
for i=1:length(trainidx)
    trainind = find(ismember( allfields,uniquefields(trainidx{i})));
    testind = find(ismember( allfields,uniquefields(testidx{i})));
    tic;
    trainlabels = alllabels(trainind);
    testlabels = alllabels(testind);
    traindata = allfeats(trainind,:);
    testdata = allfeats(testind,:);

    [traindata, testdata] = featnorm( traindata, testdata);
    traindata = double( 2*traindata-1);
    testdata = double( 2*testdata-1);
    traindata(isnan(traindata))=0;
    testdata(isnan(testdata))=0;

    idx_sda = 1:size(traindata,2);

    % For each fold, i, train a classifier on training data
    [model,options] = trainClassifier( trainlabels, traindata(:,idx_sda), c, g);

    % For each fold, i, evaluate a classifier on testing set 
    [predlabels accuracy weights] = svmpredict( testlabels,testdata(:,idx_sda), model,'-b 1');

    if ~exist('allweights','var')
        allweights = zeros(size(alllabels,1),length(UU));
    end
    allweights(testind,:) = weights(:,model.Label);

    t = toc;
    disp( ['Done with fold ' num2str(i) ' (' num2str(t) ' sec)']);
    disp(options);
end

% Relabel the data
vlabel = zeros(size(uniquefields));
vpred = zeros(size(uniquefields));
for i=1:length(uniquefields)
    idx = find(allfields==uniquefields(i));
    [a ind] = max(prod(allweights(idx,:),1),[],2);
    vlabel(i) = alllabels(idx(1));
    vpred(i) = ind;% model.Label(ind);
end

% Generate a confusion matrix
[C,U,D,W] = conmatrix( vlabel,vpred);

numims = sum(C,2);
confmat = round(1000*D)/10;
acc = round(1000*U)/10;


